import { ChainAssetId, AnyChain } from '@moonbeam-network/xcm-types';
import { SetOptional } from 'type-fest';
import { Struct, u128 } from '@polkadot/types';
import { SubmittableExtrinsicFunction } from '@polkadot/api/types';
import { ApiPromise } from '@polkadot/api';

declare enum CallType {
  Evm = 'Evm',
  Substrate = 'Substrate',
}

interface BaseConfigConstructorParams {
  module: string;
  func: string;
  type: CallType;
}
declare class BaseConfig {
  readonly module: string;
  readonly func: string;
  readonly type: CallType;
  constructor({ module, func, type }: BaseConfigConstructorParams);
}

interface QueryConfigConstructorParams
  extends SetOptional<BaseConfigConstructorParams, 'type'> {
  args?: any[];
  transform: (data: any) => Promise<bigint>;
}
declare class SubstrateQueryConfig extends BaseConfig {
  readonly args: any[];
  readonly transform: (data: any) => Promise<bigint>;
  constructor({
    args,
    transform,
    type,
    ...other
  }: QueryConfigConstructorParams);
}

interface AssetMinConfigBuilder {
  build: (params: AssetMinConfigBuilderPrams) => SubstrateQueryConfig;
}
interface AssetMinConfigBuilderPrams {
  asset: ChainAssetId;
}

declare function AssetMinBuilder(): {
  assetRegistry: typeof assetRegistry;
  assets: typeof assets$1;
};
declare function assetRegistry(): {
  assetMetadatas: () => AssetMinConfigBuilder;
  currencyMetadatas: () => AssetMinConfigBuilder;
};
declare function assets$1(): {
  asset: () => AssetMinConfigBuilder;
};

interface BalanceConfigBuilder {
  build: (params: BalanceConfigBuilderPrams) => SubstrateQueryConfig;
}
interface BalanceConfigBuilderPrams {
  address: string;
  asset: ChainAssetId;
}
interface PalletBalancesAccountDataOld extends Struct {
  readonly free: u128;
  readonly reserved: u128;
  readonly miscFrozen: u128;
  readonly feeFrozen: u128;
}
interface TokensPalletAccountData {
  free: u128;
  reserved: u128;
  frozen: u128;
}
type EquilibriumSystemBalanceData = Array<
  [
    number,
    {
      positive: number;
    },
  ]
>;

declare function BalanceBuilder(): {
  assets: typeof assets;
  system: typeof system;
  tokens: typeof tokens;
};
declare function assets(): {
  account: () => BalanceConfigBuilder;
};
declare function system(): {
  account: () => BalanceConfigBuilder;
  accountEquilibrium: () => BalanceConfigBuilder;
};
declare function tokens(): {
  accounts: () => BalanceConfigBuilder;
};

interface ContractConfigConstructorParams
  extends Omit<BaseConfigConstructorParams, 'type'> {
  args: any[];
}
declare class ContractConfig extends BaseConfig {
  readonly args: any[];
  constructor({ args, ...other }: ContractConfigConstructorParams);
}

interface ContractConfigBuilder {
  build: (params: ContractConfigBuilderPrams) => ContractConfig;
}
interface ContractConfigBuilderPrams {
  address: string;
  amount: bigint;
  asset: ChainAssetId;
  destination: AnyChain;
  fee: bigint;
  feeAsset: ChainAssetId;
}

declare function Xtokens(): {
  transfer: (weight?: number) => ContractConfigBuilder;
  transferMultiCurrencies: (weight?: number) => ContractConfigBuilder;
};

declare function ContractBuilder(): {
  Xtokens: typeof Xtokens;
};

interface ExtrinsicConfigConstructorParams
  extends Omit<BaseConfigConstructorParams, 'type'> {
  getArgs: (func?: SubmittableExtrinsicFunction<'promise'>) => any[];
}
declare class ExtrinsicConfig extends BaseConfig {
  getArgs: (func?: SubmittableExtrinsicFunction<'promise'>) => any[];
  constructor({ getArgs, ...other }: ExtrinsicConfigConstructorParams);
}

interface ExtrinsicConfigBuilder {
  build: (params: ExtrinsicConfigBuilderPrams) => ExtrinsicConfig;
}
interface ExtrinsicConfigBuilderPrams {
  address: string;
  amount: bigint;
  asset: ChainAssetId;
  destination: AnyChain;
  fee: bigint;
  feeAsset: ChainAssetId;
  palletInstance?: number;
  source: AnyChain;
}
declare enum XcmVersion {
  v1 = 'V1',
  v2 = 'V2',
  v3 = 'V3',
}
type Parents = 0 | 1;

declare function eqBalances(): {
  xcmTransfer: () => ExtrinsicConfigBuilder;
  transferXcm: () => ExtrinsicConfigBuilder;
};

declare function polkadotXcm(): {
  limitedReserveTransferAssets: () => {
    here: () => ExtrinsicConfigBuilder;
    X1: () => ExtrinsicConfigBuilder;
    X2: () => ExtrinsicConfigBuilder;
  };
  limitedReserveWithdrawAssets: () => {
    X2: () => ExtrinsicConfigBuilder;
  };
};

declare function xTokens(): {
  transfer: () => ExtrinsicConfigBuilder;
  transferMultiAsset: (originParachainId: number) => {
    X1: () => ExtrinsicConfigBuilder;
    X2: () => ExtrinsicConfigBuilder;
  };
  transferMultiCurrencies: () => ExtrinsicConfigBuilder;
};

declare function xTransfer(): {
  transfer: () => {
    here: () => ExtrinsicConfigBuilder;
    X2: () => ExtrinsicConfigBuilder;
  };
};

declare function xcmPallet(): {
  limitedReserveTransferAssets: (parents?: Parents) => {
    here: () => ExtrinsicConfigBuilder;
  };
};

declare function ExtrinsicBuilder(): {
  eqBalances: typeof eqBalances;
  xTokens: typeof xTokens;
  xTransfer: typeof xTransfer;
  polkadotXcm: typeof polkadotXcm;
  xcmPallet: typeof xcmPallet;
};

interface SubstrateCallConfigConstructorParams {
  api: ApiPromise;
  call: () => Promise<bigint>;
}
declare class SubstrateCallConfig {
  readonly api: ApiPromise;
  readonly call: () => Promise<any>;
  readonly type = CallType.Substrate;
  constructor({ api, call }: SubstrateCallConfigConstructorParams);
}

interface FeeConfigBuilder {
  build: (params: FeeConfigBuilderPrams) => SubstrateCallConfig;
}
interface FeeConfigBuilderPrams {
  asset: ChainAssetId;
  api: ApiPromise;
}

declare function FeeBuilder(): {
  assetManager: typeof assetManager;
};
declare function assetManager(): {
  assetTypeUnitsPerSecond: (weight?: number) => FeeConfigBuilder;
};

export {
  AssetMinBuilder,
  AssetMinConfigBuilder,
  AssetMinConfigBuilderPrams,
  BalanceBuilder,
  BalanceConfigBuilder,
  BalanceConfigBuilderPrams,
  BaseConfig,
  BaseConfigConstructorParams,
  ContractBuilder,
  ContractConfig,
  ContractConfigBuilder,
  ContractConfigBuilderPrams,
  ContractConfigConstructorParams,
  EquilibriumSystemBalanceData,
  ExtrinsicBuilder,
  ExtrinsicConfig,
  ExtrinsicConfigBuilder,
  ExtrinsicConfigBuilderPrams,
  ExtrinsicConfigConstructorParams,
  FeeBuilder,
  FeeConfigBuilder,
  FeeConfigBuilderPrams,
  PalletBalancesAccountDataOld,
  Parents,
  QueryConfigConstructorParams,
  SubstrateQueryConfig,
  TokensPalletAccountData,
  XcmVersion,
};
